// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Java Audio Client                                                ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


import TransportInfo;
import MediaInfo;


/**
  * This class is the native interface for AudioClient.
  *
  * @short   Java audio client
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class AudioClient
{
   // ====== Constructor/"Destructor" =======================================
   /**
     * Constructor for a new AudioClient.
     *
     * @param outputDevice 0 for AudioDevice, 1000 for AudioDebug, -1 for AudioNull.
     */
   public AudioClient(int outputDevice) throws OutOfMemoryError {
      if(constructIt(outputDevice) == false) {
         throw new OutOfMemoryError();
      }
   }

   /**
     * Do garbage collection: Delete AudioClient and AudioDevice.
     * Note: Due to some Java incompatibilities, the native method destructIt
     *       may not be called from finalize() directly. Otherwise Java will crash
     *       running destructIt() (AudioClient and AudioDevice addresses are not
     *       delivered correctly to native method!!)
     */   
   synchronized public void doFinalization() {
      destructIt();
   }

   /**
     * Do garbage collection.
     * Note: Due to some Java incompatibilities, the native method destructIt
     *       may not be called from finalize() directly. Otherwise Java will crash
     *       running destructIt() (AudioClient and AudioDevice addresses are not
     *       delivered correctly to native method!!)
     */   
   synchronized public void finalize() {
      doFinalization();
   }


   /**
     * Kill Java interpreter by sending SIGINT and SIGKILL to current process.
     * This is necessary due to Java 1.3 System.exit() bug!
     */
   static synchronized public native void killJavaInterpreter();


   // ====== AudioClient methods ============================================
   /**
     * Start playing given media from given server.
     *
     * @param server Server address (e.g. gaffel:7500).
     * @param mediaName Media name (e.g. ../AudioFiles/Test1.list)
     * @return true, if play request has been sent to server.
     */
   synchronized public native boolean play(String server, String mediaName);

   /**
     * Change media of an established connection.
     *
     * @param mediaName New media name (e.g. ../AudioFiles/Test2.list)
     *
     * @see AudioClient#play
     */
   synchronized public native void change(String mediaName);

   /**
     * Stop playing.
     */
   synchronized public native void stop();


   /**
     * Get current media position.
     *
     * @return Position in milliseconds.
     */
   synchronized public native long getPosition();

   /**
     * Get maximum media position.
     *
     * @return Maximum position in milliseconds.
     */
   synchronized public native long getMaxPosition();
 
   /**
     * Get audio sampling rate.
     *
     * @return Sampling rate.
     */
   synchronized public native int getSamplingRate();
   
   /**
     * Get number of audio channels.
     *
     * @return Number of audio channels.
     */
   synchronized public native int getChannels();

   /**
     * Get number of audio bits
     *
     * @return Number of audio bits
     */
   synchronized public native int getBits();


   /**
     * Get audio bytes per second.
     *
     * @return Number of bytes per second.
     */
   synchronized public native int getBytesPerSecond();

   /**
     * Get bits per sample.
     *
     * @return Number of bits per sample.
     */
   synchronized public native int getBitsPerSample();


   /**
     * Get audio byte order.
     *
     * @return Audio byte order.
     */
   synchronized public native int getByteOrder();


   /**
     * Get number of raw bytes (incl. IPv6/UDP/RTP/RTPAudio headers) per second.
     *
     * @return Number of raw bytes per second.
     */
   synchronized public native int getRawBytesPerSecond();


   /**
     * Get number of bytes received
     *
     * @param layer Layer number or -1 for sum of all layers.
     * @return Number of bytes received
     */
   synchronized public native long getBytesReceived(int layer);

   /**
     * Get number of packets received
     *
     * @param layer Layer number or -1 for sum of all layers.
     * @return Number of packets received
     */
   synchronized public native long getPacketsReceived(int layer);

   /**
     * Get TransportInfo for current setting.
     *
     * @param transportInfo TransportInfo.
     */
   synchronized public native void getTransportInfo(TransportInfo transportInfo);

   /**
     * Get MediaInfo.
     *
     * @param mediaInfo MediaInfo.
     */
   synchronized public native void getMediaInfo(MediaInfo mediaInfo);

   /**
     * Get error code.
     *
     * @return Error code.
     */
   synchronized public native int getErrorCode();

   /**
     * Get IP version.
     *
     * @return IP Version.
     */
   synchronized public native int getIPVersion();

   /**
     * Get encoding name.
     *
     * @return Encoding name.
     */
   synchronized public native String getEncoding();

   /**
     * Get bandwidth limit.
     *
     * @return Bandwidth limit.
     */
   synchronized public native int getBandwidthLimit();

   /**
     * Get encoding name for a given index of the client's decoder repository.
     *
     * @param index Repository index.
     * @return Encoding name or NULL, if index is too high.
     */
   synchronized public native String getEncodingName(int index);

   /**
     * Get entry from table of valid sampling rates.
     *
     * @param index Table index.
     * @return Value or NULL, if index is too high.
     */
   synchronized public native int getValidSamplingRate(int index);

   /**
     * Get entry from table of valid audio bits.
     *
     * @param index Table index.
     * @return Value or NULL, if index is too high.
     */
   synchronized public native int getValidBits(int index);

   /**
     * Check, if audio client is playing.
     *
     * @return true, if client is playing; false otherwise.
     */ 
   synchronized public native boolean playing();

   /**   
     * Get server address string.
     *
     * @return Server address.
     */
   synchronized public native String getServerAddressString();

   /**
     * Get client address string.
     *
     * @return Client address.
     */
   synchronized public native String getOurAddressString();

   /**
     * Get flow label of last received packet in given layer.
     *
     * @param layer Layer number.
     * @return Flow label.
     */
   synchronized public native int getFlowLabel(int layer);

   /**
     * Get traffic class of last received packet in given layer.
     *
     * @param layer Layer number.
     * @return Traffic class.
     */
   synchronized public native int getTrafficClass(int layer);

   /**
     * Get name for traffic class.
     *
     * @param trafficClass Traffic class.
     */
   public native String getNameForTrafficClass(int trafficClass);

   /**
     * Get number of layers in last transmission.
     */
   synchronized public native int getLayers();

   /**   
     * Get server SSRC in given layer.
     *
     * @param layer Layer number.
     * @return Server SSRC.
     */
   synchronized public native int getServerSSRC(int layer);

   /**
     * Get client SSRC.
     *
     * @return Client SSRC.
     */
   synchronized public native int getOurSSRC();

   /**
     * Get number of packets lost in given layer.
     *
     * @param layer Layer number.
     * @return Number of packets lost.
     */
   synchronized public native long getPacketsLost(int layer);

   /**
     * Get fraction of packets lost in given layer.
     *
     * @param layer Layer number.
     * @return Fraction of packets lost.
     */
   synchronized public native double getFractionLost(int layer);

   /**
     * Get jitter in given layer.
     *
     * @param layer Layer number.
     * @return Jitter.
     */
   synchronized public native double getJitter(int layer);


   /**   
     * Set media position.
     *   
     * @param position New media position in milliseconds.
     */   
   synchronized public native void setPosition(long position);

   /**   
     * Set pause.
     *   
     * @param on true for pause on; false for pause off.
     */
   synchronized public native void setPause(boolean on);

   /**   
     * Set audio sampling rate.
     *   
     * @param rate New audio sampling rate.
     */   
   synchronized public native void setSamplingRate(int rate);

   /**   
     * Set number of audio channels
     *   
     * @param channels New number of audio channels.
     */
   synchronized public native void setChannels(int channels);

   /**   
     * Set number of audio bits.
     *   
     * @param bits New number of audio bits.
     */
   synchronized public native void setBits(int bits);

   /**   
     * Set encoding by index in client's decoder repository.
     *   
     * @param index Index in decoder repository.
     */
   synchronized public native void setEncoding(int index);

   /**
     * Set bandwidth limit.
     *
     * @param bandwidthLimit Bandwidth limit.
     */
   synchronized public native void setBandwidthLimit(int bandwidthLimit);


   /**
     * Possible error codes returned by getErrorCode().
     */
   final static int ME_NoError            = 0;
   final static int ME_NoMedia            = 1;
   final static int ME_EOF                = 2;
   final static int ME_UnrecoverableError = 20;
   final static int ME_BadMedia           = ME_UnrecoverableError + 0;
   final static int ME_ReadError          = ME_UnrecoverableError + 1;
   final static int ME_OutOfMemory        = ME_UnrecoverableError + 2;

   /**
     * Constant for position steps per second: 1 step = 1 nanosecond;
     */
   final static long PositionStepsPerSecond = (long)1000000000;


   // ====== Private data ===================================================   
   /**
     * Loading native library.
     */
   static {
      System.loadLibrary("AudioClient");
   }

   private native boolean constructIt(int outputDevice);
   private native void    destructIt();

   private int  AudioClientInstance32;
   private int  AudioDeviceInstance32;
   private long AudioClientInstance64;
   private long AudioDeviceInstance64;
}
