// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Transport Info                                                   ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


import TransportInfoLayer;
import TransportInfoLevel;


/**
  * This class keeps information on an encoding's transport of data:
  * Quality levels and layers, required bandwidth, etc.
  *
  * @short   Transport Information
  * @author  Thomas Dreibholz (Dreibholz@bigfoot.com)
  * @version 1.0
  */
public class TransportInfo
{
   // ====== Constructor ====================================================
   /**
     * Constructor.
     */
   public TransportInfo() {
      MinBytesPerSecond      = new long[TransportInfoLevel.MaxQualityLayers];
      MaxBytesPerSecond      = new long[TransportInfoLevel.MaxQualityLayers];
      WantedBytesPerSecond   = new long[TransportInfoLevel.MaxQualityLayers];
      MinPacketsPerSecond    = new int[TransportInfoLevel.MaxQualityLayers];
      MaxPacketsPerSecond    = new int[TransportInfoLevel.MaxQualityLayers];
      WantedPacketsPerSecond = new int[TransportInfoLevel.MaxQualityLayers];
      WantedMaxLossRate      = new int[TransportInfoLevel.MaxQualityLayers];
      QualityLevel = new TransportInfoLevel[MaxQualityLevels];
      for(int i = 0;i < MaxQualityLevels;i++) {
         QualityLevel[i] = new TransportInfoLevel();
      }
      CurrentSetting = new TransportInfoLevel();
   }


   // ====== Reset ==========================================================
   /**
     * Reset.
     */
   public void reset() {
      for(int i = 0;i < TransportInfoLevel.MaxQualityLayers;i++) {
         MinBytesPerSecond[i]   = MaxBytesPerSecond[i]   = WantedBytesPerSecond[i]   = 0;
         MinPacketsPerSecond[i] = MaxPacketsPerSecond[i] = WantedPacketsPerSecond[i] = 0;
      }
      MinFramesPerSecond     = 0;
      MaxFramesPerSecond     = 0;
      WantedFramesPerSecond  = 0;
      WantedMaxTransferDelay = -1;
      Flags                  = 0;
      QualityLevels          = 0;
      StartFramesPerSecond   = 0;
      for(int i = 0;i < MaxQualityLevels;i++) {
         QualityLevel[i].reset();
      }
      CurrentSetting.reset();
   }


   // ====== Print ==========================================================
   /**
     * Print.
     */
   public void print() {
      System.out.println("TransportInfo:");
      for(int i = 0;i < QualityLayers;i++) {
         System.out.println("Layer #" + i + ":");
         System.out.println("   WantedBytesPerSecond   = " + MinBytesPerSecond[i] + " <= " + WantedBytesPerSecond[i] + " <= " + MaxBytesPerSecond[i]);
         System.out.println("   WantedPacketsPerSecond = " + MinPacketsPerSecond[i] + " <= " + WantedPacketsPerSecond[i] + " <= " + MaxPacketsPerSecond[i]);
         System.out.println("   WantedMaxLossRate      = " + WantedMaxLossRate[i]);
      } 
      System.out.println("WantedFramesPerSecond  = " + MinFramesPerSecond + " <= " + WantedFramesPerSecond + " <= " + MaxFramesPerSecond);
      System.out.println("WantedMaxTransferDelay = " + (WantedMaxTransferDelay / 16) + " [ms]");
      System.out.println("Flags                  = " + Flags);
      System.out.println("StartFramesPerSecond   = " + StartFramesPerSecond);
      System.out.println("QualityLevels          = " + QualityLevels);
      System.out.println("QualityLayers          = " + QualityLayers);

      for(int i = 0;i < QualityLevels;i++) {
         System.out.println("Level #" + i + ":");
         QualityLevel[i].print();
      }
      System.out.println("Current Setting:");
      CurrentSetting.print();
   }


   // ====== Constants ======================================================
   /**
     * Constant for maximum number of quality levels.
     */
   public final int MaxQualityLevels = 24;


   // ====== Wanted values ==================================================
   /**
     * Bytes per second: Minimum value for each layer.
     */
   public long MinBytesPerSecond[];

   /**
     * Bytes per second: Maximum value for each layer.
     */
   public long MaxBytesPerSecond[];

   /**
     * Bytes per second: Wanted value for each layer.
     */
   public long WantedBytesPerSecond[];


   /**
     * Packets per second: Minimum value for each layer.
     */
   public int MinPacketsPerSecond[];

   /**
     * Packets per second: Maximum value for each layer.
     */
   public int MaxPacketsPerSecond[];

   /**
     * Packets per second: Wanted value for each layer.
     */
   public int WantedPacketsPerSecond[];


   /**
     * Frames per second: Minimum value.
     */
   public int MinFramesPerSecond;

   /**
     * Frames per second: Maximum value.
     */
   public int MaxFramesPerSecond;

   /**
     * Frames per second: Wanted value.
     */
   public int WantedFramesPerSecond;


   /**
     * Wanted maximum transfer delay in 1/16 milliseconds.
     */
   public int WantedMaxTransferDelay;


   /**
     * Wanted maximum loss rate in fraction of 255 for each layer.
     */
   public int WantedMaxLossRate[];


   // ====== Flags and number of quality levels =============================
   /**
     * TransportInfo flags.
     */
   public int Flags;

   /**
     * TransportInfo flag: No flags.
     */
   public static final int TIF_None = 0;

   /**
     * TransportInfo flag: Encoding has dynamic quality settings.
     */
   public static final int TIF_HasDynamic = 1;


   // ====== Quality levels =================================================
   /**
     * Number of quality levels stored in QualityLevel array.
     * This is a constant set by the encoding.
     */
   public int QualityLevels;

   /**
     * Maximum number of quality layers within in a level. It is used to limit
     * the range of valid Wanted values.
     */
   public int QualityLayers;

   /**
     * Start frames per second.
     */
   public int StartFramesPerSecond;

   /**
     * Quality level transport information array.
     * All values are *constants* of the encoding.
     */
   public TransportInfoLevel QualityLevel[];


   // ====== Current quality setting ========================================
   /**
     * Current setting's transport information.
     * The encoding will store current setting here.
     */
   public TransportInfoLevel CurrentSetting;
};
