// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Audio Encoder Repository Implementation                          ####
// ####                                                                  ####
// #### Version 1.00  --  February 16, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "audioencoderrepository.h"
#include "audioquality.h"


namespace Coral {


// ###### Constructor #######################################################
AudioEncoderRepository::AudioEncoderRepository()
{
   Encoder = NULL;
}


// ###### Destructor ########################################################
AudioEncoderRepository::~AudioEncoderRepository()
{
   if(Encoder) {
      Encoder->deactivate();
   }
   if(AutoDelete) {
      while(AudioEncoderRepository.begin() != AudioEncoderRepository.end()) {
         multimap<const card16,AudioEncoderInterface*>::iterator encoderIterator =
            AudioEncoderRepository.begin();
         Encoder = encoderIterator->second;
         AudioEncoderRepository.erase(encoderIterator);
         delete Encoder;
      }
   }
   Encoder = NULL;   
}


// ###### Add encoder to repository #########################################
bool AudioEncoderRepository::addEncoder(AudioEncoderInterface* encoder)
{
   multimap<const card16,AudioEncoderInterface*>::iterator encoderIterator =
      AudioEncoderRepository.find(encoder->getTypeID());
   if(encoderIterator == AudioEncoderRepository.end()) {
      AudioEncoderRepository.insert(pair<const card16,AudioEncoderInterface*>
                                    (encoder->getTypeID(),encoder));
      if(Encoder == NULL) {
         Encoder = encoder;
      }
      return(true);
   }
   return(false);
}


// ###### Remove encoder from repository ####################################
void AudioEncoderRepository::removeEncoder(AudioEncoderInterface* encoder)
{
   multimap<const card16,AudioEncoderInterface*>::iterator encoderIterator =
      AudioEncoderRepository.find(encoder->getTypeID());
   if(encoderIterator != AudioEncoderRepository.end()) {
      AudioEncoderRepository.erase(encoderIterator);
      encoder->deactivate();
      if(Encoder == encoder) {
         multimap<const card16,AudioEncoderInterface*>::iterator firstEncoder =
            AudioEncoderRepository.begin();
         if(firstEncoder != AudioEncoderRepository.end()) {
            Encoder = firstEncoder->second;
         }
         else {
            Encoder = NULL;
         }
      }
      else {
         Encoder = NULL;
      }
   }
}


// ###### Select encoder for given type ID ##################################
bool AudioEncoderRepository::selectEncoderForTypeID(const card16 typeID)
{
   if((Encoder == NULL) || (typeID != Encoder->getTypeID())) {
      multimap<const card16,AudioEncoderInterface*>::iterator encoderIterator =
         AudioEncoderRepository.find(typeID);
      if(encoderIterator != AudioEncoderRepository.end()) {
          AudioEncoderInterface* encoder = encoderIterator->second;
          Encoder->deactivate();
          const AudioQuality quality(*Encoder);

          Encoder = encoder;

          Encoder->setQuality(quality);
          Encoder->activate();
          return(true);
      }
      return(false);
   }
   return(true);
}


// ###### Get pointer to current encoder ####################################
EncoderInterface* AudioEncoderRepository::getCurrentEncoder() const
{
   return((EncoderInterface*)Encoder);
}
   

// ###### Get pointer to current audio encoder ##############################   
AudioEncoderInterface* AudioEncoderRepository::getCurrentAudioEncoder() const
{
   return(Encoder);
}



/* 
#############################################################################
###### State Pattern: Method calls                                     ######
#############################################################################
*/

const card16 AudioEncoderRepository::getTypeID() const
   { return(Encoder->getTypeID()); }

const char* AudioEncoderRepository::getTypeName() const
   { return(Encoder->getTypeName()); }

void AudioEncoderRepository::activate()
   { Encoder->activate(); }

void AudioEncoderRepository::deactivate()
   { Encoder->deactivate(); }

void AudioEncoderRepository::reset()
   { Encoder->reset(); }
   
void AudioEncoderRepository::adaptQuality(const double   fractionLost,
                                          const cardinal layer)
   { Encoder->adaptQuality(fractionLost,layer); }

void AudioEncoderRepository::getTransportInfo(TransportInfo& transportInfo,
                                              const cardinal headerSize,
                                              const cardinal maxPacketSize,
                                              const bool calculateLevels) const
   { Encoder->getTransportInfo(transportInfo,headerSize,maxPacketSize,calculateLevels); }

void AudioEncoderRepository::setTransportInfo(TransportInfo& transportInfo,
                                              const cardinal headerSize,
                                              const cardinal maxPacketSize,
                                              const bool calculateLevels)
   { Encoder->setTransportInfo(transportInfo,headerSize,maxPacketSize,calculateLevels); }

bool AudioEncoderRepository::prepareNextFrame(const cardinal headerSize,
                                              const cardinal maxPacketSize,
                                              const cardinal flags)
   { return(Encoder->prepareNextFrame(headerSize,maxPacketSize,flags)); }

cardinal AudioEncoderRepository::getNextPacket(EncoderPacket* encoderPacket)
   { return(Encoder->getNextPacket(encoderPacket)); }

card8 AudioEncoderRepository::getChannels() const
   { return(Encoder->getChannels()); }

card8 AudioEncoderRepository::getBits() const
   { return(Encoder->getBits()); }

card16 AudioEncoderRepository::getSamplingRate() const
   { return(Encoder->getSamplingRate()); }

card16 AudioEncoderRepository::getByteOrder() const
   { return(Encoder->getByteOrder()); }

cardinal AudioEncoderRepository::getBytesPerSecond() const
   { return(Encoder->getBytesPerSecond()); }

cardinal AudioEncoderRepository::getBitsPerSample() const
   { return(Encoder->getBitsPerSample()); }

card8 AudioEncoderRepository::setChannels(const card8 channels)
   { return(Encoder->setChannels(channels)); }

card8 AudioEncoderRepository::setBits(const card8 bits)
   { return(Encoder->setBits(bits)); }

card16 AudioEncoderRepository::setSamplingRate(const card16 rate)
   { return(Encoder->setSamplingRate(rate)); }

card16 AudioEncoderRepository::setByteOrder(const card16 byteOrder)
   { return(Encoder->setByteOrder(byteOrder)); }


}
