// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### Network Logger                                                   ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "internetaddress.h"
#include "networkmonitorinterface.h"
#include "networkmonitor.h"
#include "thread.h"
#include "timedthread.h"
#include "tools.h"
#include "netlogwriter.h"
#include "breakdetector.h"


#include <multiset.h>
#include <algo.h>


using namespace Coral;


multiset<PortableAddress> acceptedSourceList;
multiset<PortableAddress> acceptedDestinationList;


// ###### Add address to accept list ########################################
void addAddress(const char* address, const bool source, const bool destination)
{
   InternetAddress sa(address);
   if(sa.isValid()) {
      PortableAddress pa = sa.getPortableAddress();
      if(source == true) {
         acceptedSourceList.insert(pa);
      }
      if(destination == true) {
         acceptedDestinationList.insert(pa);
      }
   }
   else {
      cerr << "ERROR: Address " << address << " is invalid -> Skipping!" << endl;
      exit(1);
   }
}


// ###### Main program ######################################################
int main(int argc, char** argv)
{
   char*    device                   = NULL;
   char*    file                     = NULL;
   card64   interval                 = 1000000;
   cardinal max                      = 0xffffffff;
   cardinal maxStreams               = 250;

   bool     optAcceptAllSources      = false;
   bool     optAcceptAllDestinations = false;
   bool     optPrintAccepted         = false;
   bool     optPrintRejected         = false;

   // ====== Get arguments ==================================================
   for(cardinal i = 1;i < (cardinal)argc;i++) {
      if(!(strncasecmp(argv[i],"-device=",8))) {
          device = &argv[i][8];
      }
      else if(!(strncasecmp(argv[i],"-file=",6))) {
          file = &argv[i][6];
      }
      else if(!(strncasecmp(argv[i],"-max=",5))) {
         max = atol(&argv[i][5]);
      }
      else if(!(strncasecmp(argv[i],"-maxstreams=",12))) {
         maxStreams = atol(&argv[i][12]);
      }
      else if(!(strncasecmp(argv[i],"-interval=",10))) {
         interval = (card64)atol(&argv[i][10]) * 1000;
      }
      else if(!(strncasecmp(argv[i],"-accept=",8))) {
         addAddress(&argv[i][8],true,true);
      }
      else if(!(strncasecmp(argv[i],"-accepts=",9))) {
         addAddress(&argv[i][9],true,false);
      }
      else if(!(strncasecmp(argv[i],"-acceptd=",9))) {
         addAddress(&argv[i][9],false,true);
      }
      else if(!(strcasecmp(argv[i],"-accept-all-sources"))) {
         optAcceptAllSources = true;
      }
      else if(!(strcasecmp(argv[i],"-accept-all-destinations"))) {
         optAcceptAllDestinations = true;
      }
      else if(!(strcasecmp(argv[i],"-print-accepted"))) {
         optPrintAccepted = true;
      }
      else if(!(strcasecmp(argv[i],"-print-rejected"))) {
         optPrintRejected = true;
      }
      else if(!(strcasecmp(argv[i],"-no-stream-monitoring"))) {
         maxStreams = 0;
      }
      else {
         cerr << "Wrong parameter: " << argv[i] << endl;
         cerr << "Usage: " << argv[0] << endl
              << "       {-device=name} {-interval=milliseconds} {-max=secs} {-file=name} "
              "{-maxstreams=streams} {-accept=host{:port}} {-accepts=sourcehost{:port}} "
              "{-acceptd=desthost{:port}} {-accept-all-sources} {-accept-all-destinations} "
              "{-print-accepted} {-print-rejected}"
              << endl;
         exit(1);
      } 
   }
   if(max < 2)
      max = 2;
   if((maxStreams < 100) && (maxStreams > 0))
      maxStreams = 100;
   if(interval < 100000)
      interval = 100000;
   if(file == NULL)
      file = "netlog.dat";


   // ====== Start network monitor ==========================================
   StreamMonitorReport* smr = NULL;
   if(maxStreams > 0) {
      smr = new StreamMonitorReport(maxStreams);
      if(smr == NULL) {
         cerr << "ERROR: Out of memory!" << endl;
         exit(1);
      }
      if(!smr->ready()) {
         cerr << "ERROR: StreamMonitorReport not ready!" << endl;
         exit(1);
      }
      smr->setAcceptedDestinationList(&acceptedDestinationList);
      smr->setAcceptedSourceList(&acceptedSourceList);
      smr->setAcceptAllSources(optAcceptAllSources);
      smr->setAcceptAllDestinations(optAcceptAllDestinations);
      smr->setPrintAccepted(optPrintAccepted);
      smr->setPrintRejected(optPrintRejected);
   }

   NetworkMonitor* monitor = new NetworkMonitor(smr,device);
   if(monitor == NULL) {
      cerr << "ERROR: Out of memory!" << endl;
      exit(1);
   }

   uid_t uid = getuid();
   setuid(uid);


   if(monitor->ready()) {
      NetLogWriter* writer = new NetLogWriter(file,interval,monitor,smr);
      if(writer == NULL) {
         cerr << "ERROR: Out of memory!" << endl;
         exit(1);
      }
      if(writer->ready()) {
         cout << "Network Logger - Copyright (C) 1999-2001 Thomas Dreibholz" << endl;
         cout << "---------------------------------------------------------" << endl;
         cout << endl;
         cout << "File:           " << file << endl;
         cout << "Device:         " << monitor->getDevice() << endl;
         cout << "Interval:       " << writer->getInterval() << " [s]" << endl;
         cout << "Max Entries:    " << max << endl;
         cout << "Max Streams:    " << maxStreams << endl;
         if(smr != NULL) {
            cout << "Accepted Source List:" << endl;
            if(acceptedSourceList.size() == 0) {
               smr->setAcceptAllSources(true);
               cout << "   Acceping *all* sources!" << endl;
            }
            else {
               multiset<PortableAddress>::iterator setIterator = acceptedSourceList.begin();
               while(setIterator != acceptedSourceList.end()) {
                  cout << "   " << InternetAddress(*setIterator) << endl;
                  setIterator++;
               }
               if(optAcceptAllSources == true) {
                  cerr << "NOTE: A source list and -accept-all-sources does not make sense."
                       << endl;
               }
            }
            cout << endl;
            cout << "Accepted Destination List: " << endl;
            if(acceptedDestinationList.size() == 0) {
               smr->setAcceptAllDestinations(true);
               cout << "   Acceping *all* destinations!" << endl;
            }
            else {
               multiset<PortableAddress>::iterator setIterator = acceptedDestinationList.begin();
               while(setIterator != acceptedDestinationList.end()) {
                  cout << "   " << InternetAddress(*setIterator) << endl;
                  setIterator++;
               }
               if(optAcceptAllDestinations == true) {
                  cerr << "NOTE: A destination list and -accept-all-destinations does not make sense."
                       << endl;
               }
            }
            cout << endl;
         }
         else {
            cout << "=> Stream monitoring disabled!" << endl;
         }
         cout << endl;

         // ====== Start threads ============================================
         installBreakDetector();
         monitor->start();

         // We do not need corrected timers.   
         writer->setTimerCorrection(0);
         writer->setFastStart(false);
         writer->start();

         // ====== Wait for break ===========================================
         cout << "Monitoring network..." << endl;
         while((!breakDetected()) && (writer->getCount() < max)) {
            Thread::delay(1000000,true);
         }

         // ====== Clean-up =================================================
         writer->stop();
         monitor->stop();
         cout << "Terminated!" << endl;
      }
      else {
         cerr << "ERROR: NetLogWriter not ready!" << endl;
         exit(1);
      }
      delete writer;
      delete monitor;
   }
   else {
      cerr << "ERROR: NetworkMonitor not ready!" << endl;
      exit(1);
   }
   return(0);
}
