// ##########################################################################
// ####                                                                  ####
// ####                      RTP Audio Server Project                    ####
// ####                    ============================                  ####
// ####                                                                  ####
// #### TCP Sender                                                       ####
// ####                                                                  ####
// #### Version 1.00  --  February 23, 2001                              ####
// ####                                                                  ####
// #### Copyright (C) 1999  Thomas Dreibholz                             ####
// ####               2000  Universitt Bonn, Abt. IV                    ####
// ####               2001  EMail: Dreibholz@bigfoot.com                 ####
// ####                     WWW:   http://www.bigfoot.com/~dreibholz     ####
// ####                                                                  ####
// ##########################################################################


#include "system.h"
#include "socket.h"
#include "internetaddress.h"
#include "timedthread.h"
#include "breakdetector.h"
#include "trafficclassvalues.h"


using namespace Coral;


class SenderThread : public Thread
{
   // ====== Constructor ====================================================
   public:
   SenderThread(Socket* socket, const cardinal blockSize);

   inline card64 getBytesSent();


   // ====== Private data ===================================================
   private:
   void run();


   Socket*  SenderSocket;
   cardinal BlockSize;
   card64   Sent;
};


// ###### Constructor #######################################################
SenderThread::SenderThread(Socket* socket, const cardinal blockSize)
   : Thread()
{
   SenderSocket = socket;
   Sent         = 0;
   BlockSize    = blockSize;
}


// ###### Get number of bytes sent ##########################################
inline card64 SenderThread::getBytesSent()
{
   synchronized();
   const card64 sent = Sent;
   unsynchronized();
   return(sent);
}


// ###### run() implementation ##############################################
void SenderThread::run()
{
   for(;;) {
      char buffer[BlockSize];
      const card64 sent = (card64)SenderSocket->write((char*)&buffer,BlockSize);

      if(sent > 0) {
         synchronized();
         Sent += sent;
         unsynchronized();
      }
   }
}


// ###### Main program ######################################################
int main(int argc, char** argv)
{
   // ====== Get arguments ==================================================
   if(argc < 2) {
      cerr << "Usage: " << argv[0] << " [Host:Port] {-tc=traffic class} {-blocksize=bytes} {-force-ipv4}"
           << endl;
      exit(1);
   }
   cardinal blockSize      = 1024;
   card8    trafficClass    = 0x00;
   bool     optForceIPv4       = false;
   for(cardinal i = 2;i < (cardinal)argc;i++) {
      if(!(strncasecmp(argv[i],"-blocksize=",11))) {
         blockSize = atol(&argv[i][11]);
      }
      else if(!(strncasecmp(argv[i],"-tc=",4))) {
         char tcString[32];
         int ok = sscanf(&argv[i][4],"%8[0-z]",(char*)&tcString);
         if(ok != 0) {
            card16 tc;
            tc = TrafficClassValues::getTrafficClassForName((char*)&tcString);
            if(tc == 0xffff) {
               ok = 0;
            }
            else {
               trafficClass = (card8)tc;
            }
         }
         if(ok == 0) {
            int tc;
            ok = sscanf(&argv[i][4],"%x",&tc);
            if(ok != 0) {
               if((tc < 0x00) || (tc > 0xff)) {
                  ok = 0;
               }
               else {
                  trafficClass = (cardinal)tc;
               }
            }
         }
         if(ok == 0) {
            cerr << "ERROR: Invalid traffic class setting!" << endl;
            exit(1);
         }
      }
      else if(!(strcmp(argv[i],"-force-ipv4"))) {
         optForceIPv4 = true;
      }
      else {
         cerr << "Usage: " << argv[0] << " [Host:Port] {-tc=traffic class} {-blocksize=bytes} {-force-ipv4}"
              << endl;
         exit(1);
      }
   }
   if(optForceIPv4) {
      if(InternetAddress::UseIPv6 == true) {
         InternetAddress::UseIPv6 = false;
         cerr << "NOTE: IPv6 support disabled!" << endl;
      }
   }
   if(blockSize < 16)
      blockSize = 16;
   if(blockSize > 16 * 1024 * 1024)
      blockSize = 16 * 1024 * 1024;


   // ====== Initialize socket ==============================================
   Socket          senderSocket(Socket::IP,Socket::TCP);
   InternetAddress receiverAddress(argv[1]);
   if((!receiverAddress.isValid()) ||
      (senderSocket.connect(receiverAddress,trafficClass) == false)) {
      cerr << "ERROR: Unable to connect to " << argv[1] << "!" << endl;
      exit(1);
   }
   InternetAddress socketAddress;
   InternetAddress peerAddress;
   senderSocket.getSocketAddress(socketAddress);
   senderSocket.getPeerAddress(peerAddress);


   SenderThread sender(&senderSocket,blockSize);
   sender.start();
   installBreakDetector();


   // ====== Print information ==============================================
   cout << "TCP Sender - Copyright (C) 1999-2001 Thomas Dreibholz" << endl;
   cout << "-----------------------------------------------------" << endl;
   cout << endl;
   
   cout << "Sender address:    " << socketAddress << endl;
   cout << "Receiver address:  " << peerAddress << endl;
   cout << "Block size:        " << blockSize << endl;
   char tcString[32];
   const char* tc = TrafficClassValues::getNameForTrafficClass(trafficClass);
   if(tc != NULL) {
      strcpy((char*)&tcString,tc);
   }
   else {
      snprintf((char*)&tcString,sizeof(tcString),"$%02x",trafficClass);
   }
   cout << "Traffic Class:     " << tcString << endl;
   cout << endl;


   // ====== Main loop ======================================================
   card64 oldValue = 0;
   card64 oldTS    = getMicroTime();
   while(!breakDetected()) {
      Thread::delay(2500000,true);
      sender.synchronized();
      const card64 sent = sender.getBytesSent();
      const card64 now  = getMicroTime();
      sender.unsynchronized();

      char str[512];
      snprintf((char*)&str,sizeof(str),
              "[Sent: %12Ld Bytes = %12Ld Bits]  [Rate: %9Ld Bytes/s = %9Ld Bits/s, %s]",
              sent, sent * 8,
              (sent - oldValue) * 1000000 / (now - oldTS),
              (sent - oldValue) * 8 * 1000000 / (now - oldTS),
              tcString);
      cout << str << endl;

      oldValue = sent;
      oldTS    = now;
   }


   // ====== Clean-up =======================================================
   sender.stop();
   cout << "Terminated!" << endl;
   return(0);
}
